#include <avr/io.h>
#define F_CPU 16000000
#include <util/delay.h>


int piniA[8] = {PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7};

inline void goForward()
{
	PORTB = 0;
	PORTB |= (1 << PB1);
	PORTB |= (1 << PB3);
}

inline void goLeft()
{
	PORTB = 0;
	PORTB |= (1 << PB3);
}

inline void goRight()
{
	PORTB = 0;
	PORTB |= (1 << PB1);
}

inline void instantLeft()
{
	PORTB = 0;
	PORTB |= (1 << PB3);
	PORTB |= (1 << PB0);
}

inline void instantRight()
{
	PORTB = 0;
	PORTB |= (1 << PB1);
	PORTB |= (1 << PB2);
}

inline unsigned char getLinePosition()
{
	int i, count = 0, sum = 0;
	
	/* check sensors */
	for (i = 0; i < 8; ++i) {
		if ((PINA & (1 << piniA[i])) != 0) {
			sum += i + 1;
			count++;
		}
	}
	
	/* 0 or pin average depending on black line position */
	return count == 0 ? 0 : sum / count;
}

int main()
{
	int set = 0;
	char lastDirection = 0;
	
	DDRA = 0x00; // direction port A: input pins
    DDRB = 0x0F; // direction port B: output pins - PB0...PB3
	
	PORTA = 0xff;
	PORTB = 0;
	
	DDRD &= ~(1 << PD6);
	PORTD |= 1 << PD6;
	
	while (1) {
		
		if ((PIND & (1 << PD6)) == 0) {
			set = (set + 1) % 2;
		}
		
		if (set == 1) {
			unsigned char lp = getLinePosition();
			/* black line is not detected */
			if (lp == 0) {
				if (lastDirection == 0) {
					instantLeft();
				} else {
					instantRight();
				}
			} else
				/* black line is on the left side */
				if (lp < 4) {
				goLeft();
				lastDirection = 0;
			} else 
				/* black line is on the right side */
				if (lp > 5) {
				goRight();
				lastDirection = 1;
			} else {
				goForward();
			}
		} else {
			PORTB = 0b00000000;
		}
	} 
	
	return 0;
}
